/* dirbench.c */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#define DIR_PREFIX	"test_dir_"

int start_bench(char *bench_type, clockid_t cl_id, struct timespec *tp)
{
	printf("Start to benchmark %s...\n", bench_type);
	return clock_gettime(cl_id, tp);
}

void stop_bench(int count, clockid_t cl_id, struct timespec *start_time)
{
	float result;
	struct timespec tp;
	int secdiff, nsecdiff;

	clock_gettime(cl_id, &tp);

	secdiff = tp.tv_sec - start_time->tv_sec;
	nsecdiff = tp.tv_nsec - start_time->tv_nsec;
//	printf("seddiff=%d, nsecdiff=%d\n", secdiff, nsecdiff);

	if(secdiff==0)
	{
		result = (float)count / (nsecdiff/1000.0/1000.0) *1000;
	}
	else
	{
		result = (float)count / (secdiff + nsecdiff/1000.0/1000.0/1000.0);
	}

	printf("Result: %f (dirs/sec)\n", result);
}

int rename_directory(int count)
{
	int i;
	char dir_name[256];
	char new_dir_name[256];
	int dir_prefix_len;
	
	dir_prefix_len = strlen(DIR_PREFIX);
	strcpy(dir_name, DIR_PREFIX);
	strcpy(new_dir_name, DIR_PREFIX);

	for(i=0;i<count;i++)
	{
		sprintf(&dir_name[dir_prefix_len], "%d\0", i);
		sprintf(&new_dir_name[dir_prefix_len], "%d\0", i+count);
		if(rename(dir_name, new_dir_name)<0)
			return -1;
	}

	return 0;
}

int remove_directory(int count)
{
	int i;
	char dir_name[256];
	int dir_prefix_len;
	
	dir_prefix_len = strlen(DIR_PREFIX);
	strcpy(dir_name, DIR_PREFIX);

	for(i=0;i<count;i++)
	{
		sprintf(&dir_name[dir_prefix_len], "%d\0", i+count);
		if(rmdir(dir_name)<0)
			return -1;
	}
	
	return 0;
}

int chmod_directory(int count)
{
	int i;
	char dir_name[256];
	int dir_prefix_len;
	
	dir_prefix_len = strlen(DIR_PREFIX);
	strcpy(dir_name, DIR_PREFIX);

	for(i=0;i<count;i++)
	{
		sprintf(&dir_name[dir_prefix_len], "%d\0", i);
		if(chmod(dir_name, 0x1A4)<0)
			return -1;
	}
	
	return 0;
}

int create_directory(int count)
{
	int i;
	char dir_name[256];
	int dir_prefix_len;
	
	dir_prefix_len = strlen(DIR_PREFIX);
	strcpy(dir_name, DIR_PREFIX);

	for(i=0;i<count;i++)
	{
		sprintf(&dir_name[dir_prefix_len], "%d\0", i);
		if(mkdir(dir_name, 0)<0)
			return -1;
	}
	
	return 0;
}

#define BENCH_COUNT	1000

int main(int argc, char *argv[])
{
	clockid_t cl_id;
	struct timespec bench_time;

	clock_getcpuclockid(getpid(), &cl_id);

	start_bench("create directory", cl_id, &bench_time);
	create_directory(BENCH_COUNT);
	stop_bench(BENCH_COUNT, cl_id, &bench_time);

//	start_bench("chmod directory", cl_id, &bench_time);
//	chmod_directory(BENCH_COUNT);
//	stop_bench(BENCH_COUNT, cl_id, &bench_time);

	start_bench("rename directory", cl_id, &bench_time);
	rename_directory(BENCH_COUNT);
	stop_bench(BENCH_COUNT, cl_id, &bench_time);

	start_bench("remove directory", cl_id, &bench_time);
	remove_directory(BENCH_COUNT);
	stop_bench(BENCH_COUNT, cl_id, &bench_time);

	return 0;
}

